/* output format: default, bit endian, litte endian */
OUTPUT_FORMAT("elf32-littlearm", "elf32-bigarm", "elf32-littlearm")
/* output architecture */
OUTPUT_ARCH(arm)
/* memory layout */
MEMORY {
	/* flash: origins at 0x0 with 32MiB size */
	FLASH	(rx)	: ORIGIN = 0x00000000, LENGTH = 32M
	/* sram: origins at 0x5c000000 with 256KiB size */
	SRAM	(rw)	: ORIGIN = 0x5c000000, LENGTH = 256K
	/* sdram: origins at 0xA0000000 with 32MiB size */
	RAM		(rwx)	: ORIGIN = 0xA0000000, LENGTH = 32M
}

/* section mapping */
SECTIONS
{
	/* match these values with the above ones
		(the used linker version does not provide
		the ORIGIN and LENGTH functions) */
	_SRAM_ORIGIN = 0x5c000000;
	_SRAM_LENGTH = 256k;
	_RAM_ORIGIN = 0xA0000000;
	_RAM_LENGTH = 32M;
	
	/* stack size for IRQ handler (and all functions in IRQ context)
		see barecrt.s */
	_IRQ_STACK_SIZE = 2k;
	/* stack size for rest of the programm
		see barecrt.s */
	_SVC_STACK_SIZE = 3k;
	
	/* overall stack size */
	_STACK_SIZE = _IRQ_STACK_SIZE + _SVC_STACK_SIZE;
	
	/* boot section: only available on flash, not copied into ram */
	.boot :
	{
		*(.vectors)
		
		. = 0x20;
		*(.vecaddrs)
		
		*(.start)
	} >FLASH

	/* text section: copied into ram and executed from there */
	.text :
	{
		__text_start = .;
		/* text input sections */
		*(.text .stub .text.* .gnu.linkonce.t.*)
		/* read only data */
		*(.rodata.*)
		*(.rodata)
		/* .gnu.warning sections are handled specially by elf32.em. */
		*(.gnu.warning)
		*(.glue_7t) *(.glue_7)
		/* keep finalize sections */
		KEEP (*(.fini))
	} >RAM AT>FLASH
	PROVIDE(__etext = .);
	PROVIDE(_etext = .);
	PROVIDE(etext = .);
	
	PROVIDE(__text_load_start = LOADADDR(.text));
	PROVIDE(__text_load_end = __text_load_start + SIZEOF(.text));
	
	/* data section: copied into ram */
	.data :
	{
		/* provide data start symbol */
		__data_start = . ;
		/* vector table */
		*(.vectable)
		/* functions executed from sram */
		*(.ramtext)
		/* data sections */
		*(.data .data.* .gnu.linkonce.d.*)
		*(.gnu.linkonce.d*)
		/* provide data end symbol */
		_edata = .;
		PROVIDE(edata = .);
	} >SRAM AT>FLASH
	
	/* provide load address of data section */
	PROVIDE(__data_load_start = LOADADDR(.data));
	/* and end address */
	PROVIDE(__data_load_end = __data_load_start + SIZEOF(.data));
	
	/* uninitialized data section, initialized with zeros.
		located after data section in SRAM*/
	.bss (NOLOAD) :
	{
		/* provide bss start symbol */
		__bss_start = .;
		__bss_start__ = .;
		/* bss sections */
		 *(.dynbss)
		 *(.bss .bss.* .gnu.linkonce.b.*)
		 *(COMMON)
		 /* provide bss end symbol */
		_bss_end__ = . ; __bss_end__ = . ;
		PROVIDE(end = .);
	} >SRAM
	
	/* stack section, initialized with STACK_TOKEN.
		located at end of sram */
	.stack (_SRAM_ORIGIN + _SRAM_LENGTH - _STACK_SIZE) (NOLOAD) :
	{
		/* provide stack start symbol */
		__stack_start = .;
		_stack = .;
		/* stack */
		*(.stack)
		/* provide stack end symbol */
		/* NOTE: do not rely on this symbol as the files do not export stack
			sections, thus __stack_end == _stack_start */
		PROVIDE(__stack_end = .);
	} >SRAM
	
	/* Stabs debugging sections.  */
	.stab          0 : { *(.stab) }
	.stabstr       0 : { *(.stabstr) }
	.stab.excl     0 : { *(.stab.excl) }
	.stab.exclstr  0 : { *(.stab.exclstr) }
	.stab.index    0 : { *(.stab.index) }
	.stab.indexstr 0 : { *(.stab.indexstr) }
	.comment       0 : { *(.comment) }
	/* DWARF debug sections.
		Symbols in the DWARF debugging sections are relative to the beginning
		of the section so we begin them at 0.  */
	/* DWARF 1 */
	.debug          0 : { *(.debug) }
	.line           0 : { *(.line) }
	/* GNU DWARF 1 extensions */
	.debug_srcinfo  0 : { *(.debug_srcinfo) }
	.debug_sfnames  0 : { *(.debug_sfnames) }
	/* DWARF 1.1 and DWARF 2 */
	.debug_aranges  0 : { *(.debug_aranges) }
	.debug_pubnames 0 : { *(.debug_pubnames) }
	/* DWARF 2 */
	.debug_info     0 : { *(.debug_info .gnu.linkonce.wi.*) }
	.debug_abbrev   0 : { *(.debug_abbrev) }
	.debug_line     0 : { *(.debug_line) }
	.debug_frame    0 : { *(.debug_frame) }
	.debug_str      0 : { *(.debug_str) }
	.debug_loc      0 : { *(.debug_loc) }
	.debug_macinfo  0 : { *(.debug_macinfo) }
	/* SGI/MIPS DWARF 2 extensions */
	.debug_weaknames 0 : { *(.debug_weaknames) }
	.debug_funcnames 0 : { *(.debug_funcnames) }
	.debug_typenames 0 : { *(.debug_typenames) }
	.debug_varnames  0 : { *(.debug_varnames) }
	/DISCARD/ : { *(.note.GNU-stack) }
}
