#!/bin/bash

vectors=$(arm-elf-nm --defined-only out | grep "t vectors$" | cut -d " " -f 1 -s)
if [ -z "$vectors" ]; then
	echo "Could not get symbol vectors"
	exit -1
fi
vectors=$((0x$vectors))
if [ -z "$vectors" -o $vectors -eq "0" ]; then
	echo "Could not get symbol vectors"
	exit -1
fi

offsetVMA=$(arm-elf-objdump -h out | grep "\.text" | xargs | cut -d " " -f 4 -s)
if [ -z "$offsetVMA" ]; then
	echo "Could not get VMA of section .text"
	exit -1
fi
offsetVMA=$((0x$offsetVMA))

offsetLMA=$(arm-elf-objdump -h out | grep "\.text" | xargs | cut -d " " -f 5 -s)
if [ -z "$offsetLMA" ]; then
	echo "Could not get LMA of section .text"
	exit -1
fi
offsetLMA=$((0x$offsetLMA))

vectors=$(($vectors - $offsetVMA + $offsetLMA))

echo Injected Vectors found at position $vectors

if [ -e ./tools/inject ]; then
	./tools/inject out.bin $vectors
else
	echo Injecttool not found, using dd
	
	echo Reading vectors and injected vectors
	dd if=out.bin of=inj.tmp bs=1 count=32 skip=$vectors 2>/dev/null
	dd if=out.bin of=vec.tmp bs=1 count=32 2>/dev/null

	if cmp inj.tmp vec.tmp >/dev/null; then
		echo Vectors match, all done
		rm -f inj.tmp vec.tmp
		exit 0
	fi
	
	echo Vectors do not match, injecting boot vector
	cp -f out.bin out.sav
	dd if=vec.tmp of=out.bin bs=1 count=32 seek=$vectors 2>/dev/null
	dd if=out.sav of=out.bin bs=1 seek=$(($vectors + 32)) \
		skip=$(($vectors + 32)) 2>/dev/null
	rm -f inj.tmp vec.tmp
	echo All done
fi

