/* PMIC Handler
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Bootloader

The Bootloader is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

The Bootloader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include <inttypes.h>
#include <os_arm_bare_registers_def.h>
#include <os_arm_bare_inithardware.h>
#include <pmic.h>
#include "pmic_interrupt.h"
#include "dmalloc.h"
#include "utils.h"
#include "config.h"
#include "image.h"
#include "timer.h"

/** \file pmic.c
 * \brief pmic interrupt
 *
 * print some statistics when button is pressed and reboot imote if it's
 *  pressed long enough
 */

/** \brief called by interrupt handler when a interrupt on the pmic occured
 *
 * reboot imote if pmic button is pressed long enough
 * \note called from <b>Interrupt context</b> */
void pmicInterrupt() {
	static bool firstPress = TRUE;
	uint8_t event;
	
	event = readPMIC(PMIC_EVENTA);
	readPMIC(PMIC_EVENTB);
	readPMIC(PMIC_EVENTC);
	readPMIC(PMIC_EVENTA);
	readPMIC(PMIC_EVENTB);
	readPMIC(PMIC_EVENTC);
	
	GEDR(1) = GPIO_BIT(1);
	
	if (event == 1) {
		/* button is pressed */
		if (firstPress) {
			/* set button timer */
			timerSet(4, 3, FALSE, TRUE, FALSE);
			timerStart(4, 0);
		} else {
			if (timerGet(4) > REBOOT_TIMEOUT) {
				/* button was pressed > TIMEOUT sec., reboot */
				debug("Rebooting device...");
				OWER = 1;
				OSMR3 = OSCR0 + 0x1000;
				while (1);
			} else {
#ifdef DEBUG_MEMORY
				/* button was pressed < TIMEOUT sec., debug memory */
				memoryInfo();
#endif
				if (BOOT_TIMEOUT != 0) {
					debug("Stopping Boot Timeout");
					utilStopBootTimeout();
				}
			}
		}
		firstPress = !firstPress;
	}
}

