/* USB Driver: Global (static) Configuration
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Bootloader

The driver contains ideas taken from the linux gadget device driver,
licenced under the GPL.

The Bootloader is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

The Bootloader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef USB_CONFIG_H
#define USB_CONFIG_H

/** \file usb_config.h
 * \brief USB Subsystem Configuration
 * \note Static Configuration. usb_configuration.h is the interface for the
 *  dynamic (i.e. runtime) configuration
 */

/** \addtogroup usbconfig
 * @{
 */

/** \brief max number of interfaces per configuration
 * \note max 8 */
#define USB_CONFIG_MAX_INTERFACES_PER_CONFIGURATION 1
/** \brief max number of altsettings per interface
 * \note max 8
 * \note Due to a bug in the PXA27x processor, altsettings cannot be used,
 *  thus, set this only to one */
#define USB_CONFIG_MAX_ALTSETTINGS_PER_INTERFACE 1
/** \brief max number of endpoints per altsetting
 * \note max 16 */
#define USB_CONFIG_MAX_ENDPOINTS_PER_ALTSETTING 16

/** \cond INTERNAL */

/** \brief number of strings which can be used in the USB descriptors
 * \note max 255 */
#define USB_CONFIG_MAX_STRINGS 20

/** \brief max number of overall endpoints
 * \note including endpoint zero
 * \note limited by 24 */
#define USB_CONFIG_NUM_ENDPOINTS 23 + 1

/** \endcond */

/** @} */

#endif

