/* USB Driver: External Interface for request handling
   Copyright (c) 2010 Networked Systems Group, University of Kaiserslautern

This file is part of the Imote2 Bootloader

The driver contains ideas taken from the linux gadget device driver,
licenced under the GPL.

The Bootloader is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

The Bootloader is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#ifndef USB_REQUEST_H
#define USB_REQUEST_H

#include <inttypes.h>

/** \addtogroup usbcomm
 * @{
 */
 
/** \file usb_request.h
 * \brief USB Request
 */

struct UsbRequest;
struct UsbEndpoint;

/** \brief USB Request status */
enum UsbRequestStatus {
	USB_REQUEST_STATUS_SUCCESS,
	USB_REQUEST_STATUS_CANCELED,
	USB_REQUEST_STATUS_RECONFIGURE,
	USB_REQUEST_STATUS_RESET,
	USB_REQUEST_STATUS_SHUTDOWN,
	USB_REQUEST_STATUS_STALLED,
	USB_REQUEST_STATUS_PROTOCOL
};

/** \brief request callback definition */
typedef void (*UsbRequestCallback)(struct UsbRequest *);

/** \brief USB Request */
struct UsbRequest {
	/** \brief data buffer */
	uint8_t * data;
	/** \brief length */
	uint32_t length;
	/** \brief actual length */
	uint32_t actual;
	
	/** \brief internal state */
	bool inUse;
	
	/** \brief callback state */
	enum UsbRequestStatus status;
	
	/** \brief callback to be called after request is done */
	UsbRequestCallback callback;
	
	/** \brief endpoint which the request is queued on */
	struct UsbEndpoint * ep;
	
	/** \brief next request */
	struct UsbRequest * next;
};

void usbRequestFill(struct UsbRequest * req, struct UsbEndpoint * ep,
	void * data, uint32_t length, UsbRequestCallback callback);

bool usbRequestEnqueue(struct UsbRequest * req);

void usbRequestCancel(struct UsbRequest * req);


/** @} */

#endif

